/***************************************************************************
                          graficos_lib.h  -  description
                             -------------------
    begin                : Thu Feb 8 2001
    copyright            : (C) 2001 by Ricardo Markiewicz
    email                : gazerzoser@ciudad.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _GRAFICOS_LIB
#define _GRAFICOS_LIB

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <SDL.h>
#include <SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_mixer.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>

#include "matematica.h"
#include "constantes.h"

#define ABS(a) ((a)<0?(a):(-1*(a)))
#define DELTA 0.001

//! Estado actual del sonido
/*! Flags que indican el modo actual del sonido del engine */
typedef enum {
		VOLUMEN_OFF,	/*!< Volumen desactivado */
		VOLUMEN_ON		/*!< Volumen activado */
} t_Volumen;

//! Tipo de mesage de debugging
typedef enum {
	CRITICO,		/*!< Error critico que puede causar mal funcionamiento */
	WARNING,		/*!< Aviso de algun error de mediana importancia */
	COMENTARIO	/*!< Comentario del estado de algn componente del engine */
} t_Mensage;

//! Inicia el engine del juego
/*!
	Esta es una de las funciones fundamentales del Engine, debe ser la primera
en llamarse ya que crea el entorno y los recursos necesarios para el correcto
funcionamiento del juego.
	\param w ancho de la pantalla en pixels (640 recomendado)
	\param h alto de la pantalla en pixels (480 recomendado)
	\param bpp profundidad de colores (16 bits recomendado)
	\param fullscreen boleano que indica si iniciar en modo pantalla completa
	\return 0 si todo esta bien. Otro valor en caso de error
*/
int Gra_Init(int w, int h, int bpp, bool fullscreen);

//! Convierte una superficie al formato de pantalla actual
/*!
	Debido a lo lento que es copiar imagenes de distintos formatos, es recomendable
transformar todas la imagenes cargadas al formato actual de pantalla.
*/
SDL_Surface *Gra_ConvertirFormato(SDL_Surface *);

//! Regresa el color correspondiente a las componentes (r,g,b) del formato actual
Uint32 Gra_GetColor(Uint8 r, Uint8 g, Uint8 b);

//! Regresa el color correspondiente a las componentes (r,g,b,a) del formato actual
Uint32 Gra_GetColorA(Uint8 r, Uint8 g, Uint8 b, Uint8 a);

//! Regresa la superficie que representa la pantalla del video
SDL_Surface *Gra_Screen();

//! Regresa una copia de la pantalla
SDL_Surface *Gra_CopiarPantalla();

//! Libera una superficie cargada en memoria
void Gra_FreeSurface(SDL_Surface *);

//! Retorna el ancho de la pantalla en pixels
Uint16 Gra_ScreenWidth();

//! Retorna el alto de la pantalla en pixels
Uint16 Gra_ScreenHeight();

//! Oculta el cursor de la pantalla
void Gra_HideCursor();

//! Muestra el cursor en pantalla
void Gra_ShowCursor();

//! Pone el cursor en la posicion (x,y) de la pantalla
void Gra_SetCursorPos(int x, int y);

//! Retorna la posicion (x,y) del cursor en pantalla
void Gra_GetCursorPos(int *x, int *y);

//! Regresa el ancho de una imagen cargada la lista de imagenes
/*! \see Gra_LoadImage */
Uint16 Gra_GetImagenWidth(int cod);

//! Regresa el alto de una imagen cargada la lista de imagenes
/*! \see Gra_LoadImage */
Uint16 Gra_GetImagenHeight(int cod);

//! Carga una imagen en memoria
/*!
	Esta funcion carga la imagen desde un archivo y la acumula en la lista
	de imagenes del engine. De esta manera cada imagen es cargada en memoria
	una sola vez.
	\param filename nombre del archivo a cargar
	\param alpha flag que indica si la imagen contiene canal de transparencia
	\return el indice de la imagen en memoria, o -1 en caso de error
*/
int Gra_LoadImage(char *filename, bool alpha=false);

//! Descarga una imagen en memoria
void Gra_UnloadImage(char *filename);

//! Setea el color a utiliar como mascara en una imagen
void Gra_SetColorKey(int cod, Uint8 r, Uint8 g, Uint8 b);

//! Setea el nivel de transparencia de una imagen
void Gra_SetAlpha(int, Uint32, Uint8);

//! Dibuja una imagen en la pantalla
/*! Esta funcion se encarga de dibujar una imagen en la pantalla.
	\param s superficie a copiar
	\param from region de origen
	\param pos punto de destino
*/
void Gra_BlitImagen(SDL_Surface *s, SDL_Rect *from, Punto2D pos);
void Gra_BlitImagen(int cod, SDL_Rect *from, Punto2D pos);
void Gra_BlitImagen(int cod, SDL_Rect *from, SDL_Rect *pos);
void Gra_BlitImagen(SDL_Surface *, SDL_Rect *from, SDL_Rect *pos);
void Gra_SetPixel(int x, int y, Uint32 color);
void Gra_SetPixelA(int x, int y, Uint32 color, Uint8 alpha);
Uint8 *Gra_GetPixel(int x, int y);
Uint32 Gra_GetPixelColor(int x, int y);
void Gra_Close();
void Gra_Lock();
void Gra_Unlock();
void Gra_Clean();
void Gra_Update();
void Gra_SaveScreen();
int Gra_ToggleFullScreen();
void Gra_InitJoystick();
void Gra_BlurImagen(SDL_Surface *s);
void Gra_DrawLine(int x1, int y1, int x2, int y2, Uint32 color);
void Gra_DrawBox(int x, int y, int w, int h, Uint32 fg=0xFFFFFF, Uint32 bg=0, bool filled=false);
void Gra_DrawBox3D(int x, int y, int w, int h, Uint32 fg=0xFFFFFF, Uint32 bg=0, bool filled=false, int type=T3D_IN);
Uint8 Gra_GetRed(Uint32 color);
Uint8 Gra_GetGreen(Uint32 color);
Uint8 Gra_GetBlue(Uint32 color);
Uint8 Gra_GetAlpha(Uint32 color);
/* algunas cositas en las que trabajo */
/* chiches graficos */
SDL_Surface *Gra_RotateImagen(SDL_Surface *s, float angle, bool alpha=false);
void Gra_BlitRotateImage(int cod, float angulo, Punto2D pos, bool soft, bool alpha=false);

/* Aca estan las funciones de manejo de Fonts e imprimir en pantalla */
void Gra_LoadFonts(); /* carga las fuentes */
void Gra_Print(int x, int y, int f, SDL_Color *color, int max_w, bool sombra, const char *fmt, ...);
void Gra_FreeFonts(); /* libera fuentes */
int Gra_TextWidth(int, char *);
int Gra_TextHeight(int, char *);

/* funciones de independencia de FPS y velocidad de la maquina */
void Gra_InitFPS(int _fps=60);
void Gra_SetSpeedFactor();
float Gra_SpeedFactor();
float Gra_FPS();

/* Manejo de Sonido */
int Gra_LoadWav(char *s);
int Gra_LoadMp3(char *s);
void Gra_UnloadWav(int cod);
void Gra_UnloadMp3(int cod);
void Gra_PlayWav(int cod, int times=-1);
void Gra_PlayMp3(int cod);
void Gra_StopWav(int cod);
void Gra_StopMp3(int cod);
void Gra_PauseMp3(int cod);
void Gra_VolumenMp3(int cod, int vol);
void Gra_VolumenWav(int cod, int vol);

/* Funciones de configuracion */
void Gra_Flush(); /* Guarda todas las configuraciones en disco */
void Gra_setVolumen(int son, int mus);
void Gra_getVolumen(int *son, int *mus);
void Gra_setKeys(int i, int tipo, SDLKey up, SDLKey down, SDLKey left, SDLKey right, SDLKey fire1, SDLKey fire2);
void Gra_getKeys(int i, int *tipo, SDLKey *up, SDLKey *down, SDLKey *left, SDLKey *right, SDLKey *fire1, SDLKey *fire2);
void Gra_setVolumenState(t_Volumen v);
t_Volumen Gra_getVolumenState();

/* Funciones Debug */
void Gra_Debug(const char *, const char *,const char *, t_Mensage);

/* Soporte Internacional */
void Gra_Idioma_Init(const char *);
void Gra_Idioma_Free();
char *Gra_Idioma_Name();
char *Gra_Idioma_File();
char *Gra_Idioma_Get(unsigned int i);


#endif